/*
  Copyright		(c)2011 Paolo Lavacchini (All rights reserved)
  Date:         8/2/2011
  Projet name: 	LAB 
  Site          www.lsoft.it
  Contact:      p.lavacchini@tin.it
  Subject:      open source for Arduino L_soft Arduino Basic
  Status:		Preliminary
  Version:		0.1 (10/03/2011)

  Modified:		none
  Conributors:	none

  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version. 

  This program is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
  GNU General Public License for more details. 

  You should have received a copy of the GNU General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/>. 

*/



#include "LAB_Def.h"




#ifdef ON_BOARD  //===== Arduino enviroment =======================




ISR(BADISR_vect,ISR_BLOCK)
{
	return; 
}



ISR (TIMER1_COMPA_vect,ISR_BLOCK)
{
#ifdef FLASH_LED
	if ( Tim_1_Counter & 0x10 ) // 640 mS frequency
	{
#ifdef ARDUINO_UNO
	PORTB |= 0x20;
#elif
	PORTC |= 0x40;
	PORTB |= 0x80;
#endif
	}
	else
	{
#ifdef ARDUINO_UNO
	PORTB &= ~0x20;
#elif
	PORTC &= ~(0x40);
	PORTB &= ~(0x80);
#endif
	}
#endif

	Tim_1_Counter++;
	TCNT1H = 0;		// Reinit to zero
 	TCNT1L = 0;
	return;
}


/* Interrupt recv. handle */
#ifdef ARDUINO_UNO
ISR(USART_RX_vect,ISR_BLOCK)
#endif
#ifdef ARDUINO_MEGA
ISR(USART0_RX_vect,ISR_BLOCK)
#endif
{

	Ser_Rx_Buffer[Ser_Rx_Wr_pnt] = UDR0; // Read the received data
	/* Calculate buffer index */
	if ( Ser_Rx_Cnt < SER_BUF_SIZE ) // Check overflow
	{
        Ser_Rx_Wr_pnt = ( Ser_Rx_Wr_pnt + 1 ) & SER_BUF_MASK;
        Ser_Rx_Cnt++;
        }
        else
	{
	/* Receive buffer overflow and now?*/
	}

}



#ifdef ARDUINO_UNO
ISR(USART_UDRE_vect,ISR_BLOCK)
#endif
#ifdef ARDUINO_MEGA
ISR(USART0_UDRE_vect,ISR_BLOCK)
#endif
{

        if ( !Ser_Tx_Cnt ) 
      { UCSR0B &= ~(1<<UDRIE0); return; } /* Disable UDRE interrupt */

        UDR0 = Ser_Tx_Buffer[Ser_Tx_Rd_pnt];
        Ser_Tx_Cnt--;
 	Ser_Tx_Rd_pnt = ( Ser_Tx_Rd_pnt + 1 ) & SER_BUF_MASK;
        if ( !Ser_Tx_Cnt ) UCSR0B &= ~(1<<UDRIE0); // No other interrupt
}




void Timer_1_Init (void)
{

// Timer 1 settings

	Tim_1_Counter = 0;

	TCNT1H = 0; 	// Init to zero
	TCNT1L = 0;
	TCCR1A = 0;
	TCCR1B = 3; 	// 3 = clock prescaler 64 = 4 uS
	TCCR1C = 0;
	TIMSK1 = 2; 	// compare IE
	TIFR1  = 2; 	// compare

	OCR1AH = (unsigned char)(TIM1_RATE >> 8);
	OCR1AL = (unsigned char)(TIM1_RATE & 0xff);


}




int Serial_Init (unsigned long baudrate, char stop,char parity, char mode )
{

	// Set the baud rate
	if ( mode == MODE_1 ) 
	{
	UCSR0A = 0;
        baudrate = (CPU_XTAL / 8 / baudrate - 1) / 2;
	}
	else if ( mode == MODE_2 ) 
	{
	UCSR0A = ( 1 << U2X0 );
        baudrate = (CPU_XTAL / 4 / baudrate - 1) / 2;
	} 
	else return -1;


	UBRR0H = (unsigned char) (baudrate>>8);
	UBRR0L = (unsigned char) baudrate;

	UCSR0C = 0;
	// Set frame format: data stop
	switch ( parity )
	{
	case PARITY_ODD:	UCSR0C |= (1<<UPM01); break;
	case PARITY_EVEN:	UCSR0C |= (1<<UPM00)|(1<<UPM01); break;
	case NO_PARITY:	/* no set */ break;
	default: return -1;
	}

	if ( stop == STOP_2 ) UCSR0C |= (1<<USBS0);     
	else if ( stop != STOP_1 ) return -1;

	// Forza 8 bit
	UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00);     

	// Enable UART receiver and transmitter
	UCSR0B = ( ( 1 << RXCIE0 ) | ( 1 << RXEN0 ) | ( 1 << TXEN0 ) );

	Ser_Rx_Rd_pnt=0;
	Ser_Rx_Wr_pnt=0;
	Ser_Tx_Rd_pnt=0;
	Ser_Tx_Wr_pnt=0;
	Ser_Rx_Cnt = 0;
	Ser_Tx_Cnt = 0;
	Rx_Buf_Pnt = 0;

	return 0;
}


/* Read and write functions */
unsigned char Serial_Rx_Char ( void )
{
unsigned char c;

// Bug volatile register in GCC
asm("	PUSH	R20");
asm("WRX_CHAR:");
asm("   LDS       R20,Ser_Rx_Cnt");
asm("	TST       R20 ");
asm("	BREQ      WRX_CHAR");  
asm("	POP 	 R20");

	cli();
	c = Ser_Rx_Buffer[Ser_Rx_Rd_pnt];
	Ser_Rx_Rd_pnt = ( Ser_Rx_Rd_pnt + 1 ) & SER_BUF_MASK;
	Ser_Rx_Cnt--;
	sei();
	return (c);

}




void Serial_Tx_Char ( unsigned char data )
{
// Bug volatile register in GCC
asm volatile ("	PUSH	R20");
asm volatile ("WTX_CHAR:");
asm volatile (" LDS     R20,Ser_Tx_Cnt");
// Occhio � rientrante -1!
asm volatile ("	CPI		R20, %0" :: "i" (SER_BUF_SIZE-1) );
asm volatile ("	BRGE    WTX_CHAR");  
asm volatile ("	POP 	R20");


	if ( data == 0xa ) Serial_Tx_Char(0xd);

	cli();
	Ser_Tx_Buffer[Ser_Tx_Wr_pnt] = data;
	Ser_Tx_Wr_pnt = ( Ser_Tx_Wr_pnt + 1 ) & SER_BUF_MASK;
	Ser_Tx_Cnt++;

	if ( Ser_Tx_Cnt == 1 ) UCSR0B |= (1<<UDRIE0); // First txmt
	sei();
	return;

}







unsigned char Serial_Sense_Rx ( void )
{
	return ( Ser_Rx_Cnt ); /* Return 0 if the receive buffer is empty */
}






void A_Board_Init (void)
{
	
// PB7 already used Arduino
// PG5 RED ready Shield Aeroquad
// PC6 GREEN ready Shield Aeroquad

#ifdef ARDUINO_MEGA
	DDRA = 0;
#endif

	DDRB = 0;
	DDRC = 0;
	DDRD = 0;

#ifdef ARDUINO_MEGA
	DDRK = 0;
#endif


#ifdef FLASH_LED
#ifdef ARDUINO_MEGA
	DDRC = 0x40;//test led Shield V2
	DDRB = 0x80;//test led Arduino
#endif

#ifdef ARDUINO_UNO
      DDRB = 0x20;
#endif

#endif
    
    
	cli();
#ifdef ARDUINO_UNO
	Serial_Init( 19200L,STOP_2,NO_PARITY,MODE_2 );   /* Set the baudrate to 19,200 bps */
#endif

#ifdef ARDUINO_MEGA
	Serial_Init( 115200UL,STOP_2,NO_PARITY,MODE_2 );   /* Set the baudrate to 19,200 bps */
#endif
	Timer_1_Init();
	sei();			/* enable interrupts */

	return;
}




unsigned long A_clock (void)
{
unsigned int temp;

	cli();
	temp = (Tim_1_Counter*TIM1_STEP) + (((TCNT1H<<8)+TCNT1L) * TIM1_MULT);
	sei();
	return temp;
}



#define MAX_PRINT 64
int A_printf(char *fmt,...)
{
va_list  argptr;
char str[MAX_PRINT];
int cnt,i;
 
	va_start( argptr, fmt );
	cnt = vsnprintf( str, MAX_PRINT, fmt, argptr );
	i=0; while ( str[i] ) Serial_Tx_Char(str[i++]);
	va_end( argptr );
	return( cnt );
}





int A_getch (void)
{
	return (Serial_Rx_Char());
}



void A_putc (char chr)
{
	Serial_Tx_Char(chr);
}



int A_sense_char (void)
{

	if ( !Ser_Rx_Cnt ) return -1;
	return (Ser_Rx_Buffer[Ser_Rx_Rd_pnt]);

}


void Set_Reg_AVR (int reg,unsigned char value)
{
  _SFR_MEM8(reg)=value;
}



unsigned char Get_Reg_AVR (int reg)
{
  return( _SFR_MEM8(reg));
}



void Delay_1mS (void)
{
unsigned long loct;

 loct = A_clock();
 while ( loct+1000 > A_clock() );
 return;
}




void Write_E2PROM (unsigned int address, unsigned char data)
{
	/* Wait for completion of previous write */
	while(EECR & (1<<EEPE));

	/* Set up address and Data Registers */
	EEARH = address>>8;
	EEARL = address&0xff;

	EEDR = data;

	/* Write logical one to EEMPE */
	EECR |= (1<<EEMPE);

	/* Start eeprom write by setting EEPE */
	EECR |= (1<<EEPE);
}



unsigned char Read_E2PROM (unsigned int address)
{
	/* Wait for completion of previous write */
	while(EECR & (1<<EEPE));

	/* Set up address register */
	EEARH = address>>8;
	EEARL = address&0xff;

	/* Start eeprom read by writing EERE */
	EECR |= (1<<EERE);

	/* Return data  */
	return EEDR;
}


void Save (void)
{
int ep,pp;

	if ( Mem_size > MAX_E2PROM )
	{ A_printf("Program too big\n"); return; }

	Write_E2PROM(0,0); //Default not autorun

	Write_E2PROM(1,Mem_size&0xff); //Salva M_size
	Write_E2PROM(2,Mem_size>>8);

	pp=0;
	for (ep=3; ep<Mem_size+1; ep++) Write_E2PROM(ep,Program[pp++]);

}

char Get_Auto_Run (void)
{
	Auto_Run=Read_E2PROM(0);
	if ( Auto_Run == AUTO_RUN ) return -1;
	return 0;
}

void Load (void)
{
int ep,pp;

	Auto_Run=Read_E2PROM(0);

	Mem_size=0;
	Mem_size = Read_E2PROM(1); //Salva M_size
	Mem_size |= Read_E2PROM(2) << 8;

	if ( Mem_size <= 0 || Mem_size > MAX_E2PROM )
	{ A_printf("No or invalid program\n"); return; }


	pp=0;
	for (ep=3; ep<Mem_size+1; ep++) 
		Program[pp++] = Read_E2PROM(ep);

}







#endif







